#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Daemon for Bookmarks scope of Web History Lens
"""

import sys
import os
import webbrowser
import sqlite3
import json
from gi.repository import GLib, GObject, Gio, Unity

MAX = 25
BUS_NAME = "net.launchpad.scope.webhistory.bookmarks"

GROUP_RECENT = 0
GROUP_POPULAR = 1
GROUP_BOOKMARKS = 2
GROUP_SEARCH = 3
GROUP_INTERNET = 4


class Daemon:
    """ Bookmarks daemon searches Firefox & Chromium bookmarks for results
    matching query
    """
    def __init__(self):
        """ Sets up the lens and default scope (which searches Zeitgeist for web history events)
        The lens has 4 categories:
          recently visited websites
          most popular websites
          bookmarks
          internet search results
        and 1 filter with 4 options
          last visited
            last day
            last week
            last month
            last year
        """
        self.bookmarks = []
        self.i = 0
        self.get_bookmarks_from_firefox()
        self.get_bookmarks_from_chromium()

        self.scope = Unity.Scope.new("/net/launchpad/scope/webhistory/bookmarks")
        self.scope.props.search_in_global = True
        self.scope.connect("search-changed", self.on_search_changed)
        self.scope.connect("filters-changed", self.on_filters_changed)
        self.scope.connect("notify::active", self.on_lens_active)
        self.scope.connect("activate-uri", self.activate_uri)
        self.scope.export()

    def get_bookmarks_from_chromium(self):
        """ Gets a list of bookmarks from the user's chromium profile
        """

        # Build Chromium's profile path
        path = os.getenv("HOME") + "/.config/chromium/Default/Bookmarks"

        if os.path.exists(path):
            try:
                bookmark_file = open(path)
                bookmark_json = json.load(bookmark_file)
                bookmarks = []
                for page in bookmark_json['roots']['bookmark_bar']['children']:
                    bookmark = []
                    bookmark.append(page['name'])
                    bookmark.append(page['url'])
                    bookmarks.append(bookmark)
                self.bookmarks += bookmarks
            except:
                pass

    def get_bookmarks_from_firefox(self):
        """ Gets a list of bookmarks from the user's firefox profile
        """
        # Build Firefox's profile path
        firefox_dbfile = ""
        path = os.getenv("HOME") + "/.mozilla/firefox/"
        for folder in [default for default in os.listdir(path) if default.endswith(".default")]:
            firefox_dbfile = path + folder + "/places.sqlite"

        if os.path.exists(firefox_dbfile):
            try:
                sqlite_query = '''SELECT moz_bookmarks.title, moz_places.url
                               FROM moz_bookmarks, moz_places
                               WHERE moz_places.id = moz_bookmarks.fk
                                   AND moz_bookmarks.title is not null
                                   AND moz_bookmarks.type = 1
                               ORDER BY moz_bookmarks.lastModified;'''

                conn = sqlite3.connect(firefox_dbfile)
                connection = conn.cursor()
                connection.execute(sqlite_query)
                self.bookmarks = connection.fetchall()
                connection.close()
            except:
                pass

    def on_filters_changed(self, *_):
        """ Called when a filter is clicked.  Queue's a new search
        """
        self.scope.queue_search_changed(Unity.SearchType.DEFAULT)

    def on_lens_active(self, *_):
        """ Called when the lens is activated.  Queue's a new search
        """
        if self.scope.props.active:
            self.scope.queue_search_changed(Unity.SearchType.DEFAULT)

    def on_search_changed(self, scope, search=None, search_type=0, cancellable=None):
        """ Called when the search is changed.  Gets the search string and search
        type and passes it to update_results_model()
        Args:
          scope: a scope object
          search: the search string
          search_type: the search type (global or local)
          cancellable: cancellable object
        """
        if hasattr(search, "props"):
            search_string = search.props.search_string
        else:
            search_string = ""

        if search_type == Unity.SearchType.DEFAULT:
            results = scope.props.results_model
        else:
            results = scope.props.global_results_model

        print "Search changed to: '%s'" % search_string
        self.update_results_model(search_string, results)
        if hasattr(search, "finished"):
            search.finished()

    def update_results_model(self, search, model):
        """ Clears the model of existing results, then performs a Zeitgeist search for the terms
        for the most recent websites visited and the most popular websites visited
        Args:
          search: the search terms
          model: the results model
        """
        icon = Gio.ThemedIcon.new("gtk-about").to_string()
        self.i = 0
        model.clear()
        group = GROUP_BOOKMARKS
        if model == self.scope.props.global_results_model:
            group = GROUP_INTERNET

        for bookmark in self.bookmarks:
            # Stop Firefox's smart bookmark folders from showing up
            if not bookmark[1].find("place:") == -1:
                continue

            # Search bookmark names for matches
            if not bookmark[0].lower().find(search.lower()) == -1:
                title = bookmark[0]
                uri = bookmark[1]
                if self.i < MAX:
                    model.append(uri, icon, group, "text/html", title, uri, uri)
                    self.i = self.i + 1
                    continue

            # Search bookmark urls for matches
            if not bookmark[1].lower().find(search.lower()) == -1:
                title = bookmark[0]
                uri = bookmark[1]
                if self.i < MAX:
                    model.append(uri, icon, group, "text/html", title, uri, uri)
                    self.i = self.i + 1

    def activate_uri(self, scope, uri):
        """ Open the url in the default webbrowser
        Args:
          uri: The url to be opened
        """
        webbrowser.open(uri)
        return Unity.ActivationResponse(handled=Unity.HandledType.HIDE_DASH, goto_uri='')

if __name__ == "__main__":
    SESSION_BUS_CONNECTION = Gio.bus_get_sync(Gio.BusType.SESSION, None)
    SESSION_BUS = Gio.DBusProxy.new_sync(SESSION_BUS_CONNECTION, 0, None,
                                         'org.freedesktop.DBus',
                                         '/org/freedesktop/DBus',
                                         'org.freedesktop.DBus', None)
    RESULT = SESSION_BUS.call_sync('RequestName',
                                   GLib.Variant("(su)", (BUS_NAME, 0x4)),
                                   0, -1, None)
    RESULT = RESULT.unpack()[0]

    if RESULT != 1:
        print >> sys.stderr, "Failed to own name %s. Bailing out." % BUS_NAME
        raise SystemExit(1)
    DAEMON = Daemon()
    GObject.MainLoop().run()
