#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
Daemon for Web History Lens & Zeitgeist scope
"""

import sys
from gi.repository import GLib, GObject, Gio, Unity
from zeitgeist import client, datamodel
import webbrowser
import urlparse
import gettext

_ = gettext.gettext
MAX = 25

#
# The primary bus name we grab  * must *  match what we specify in our .lens file
#
BUS_NAME = "net.launchpad.Lens.webhistory"

# These category ids must match the order in which we add them to the lens
GROUP_MOSTRECENT = 0
GROUP_MOSTPOPULAR = 1
GROUP_BOOKMARKS = 2
GROUP_SEARCH_RESULTS = 3
GROUP_INTERNET = 4


class Daemon:
    """ Web History Daemon sets up the lens & searches Zeitgeist for results
    matching query
    """
    def __init__(self):
        """ Sets up the lens and default scope(which searches Zeitgeist for web history events)
        The lens has 4 categories:
          recently visited websites
          most popular websites
          bookmarks
          internet search results
        and 1 filter with 4 options
          last visited
            last day
            last week
            last month
            last year
        """
        self.i = 0

        self.lens = Unity.Lens.new("/net/launchpad/lens/webhistory", "webhistory")
        self.scope = Unity.Scope.new("/net/launchpad/lens/webhistory/scope/webhistory")

        self.lens.props.search_hint = "Search Internet History"
        self.lens.props.visible = True
        self.lens.props.search_in_global = True

        # Populate categories
        cats = []
        cats.append(Unity.Category.new(_("Recently Visited Websites"),
                                       Gio.ThemedIcon.new("/usr/lib/unity-lens-web-history/globe.svg"),
                                       Unity.CategoryRenderer.HORIZONTAL_TILE))
        cats.append(Unity.Category.new(_("Most Popular Websites"),
                                       Gio.ThemedIcon.new("/usr/lib/unity-lens-web-history/globe.svg"),
                                       Unity.CategoryRenderer.HORIZONTAL_TILE))
        cats.append(Unity.Category.new(_("Bookmarks"),
                                       Gio.ThemedIcon.new("/usr/lib/unity-lens-web-history/globe.svg"),
                                       Unity.CategoryRenderer.HORIZONTAL_TILE))
        cats.append(Unity.Category.new(_("Internet Search Results"),
                                       Gio.ThemedIcon.new("/usr/lib/unity-lens-web-history/globe.svg"),
                                       Unity.CategoryRenderer.HORIZONTAL_TILE))
        cats.append(Unity.Category.new(_("Internet History"),
                                       Gio.ThemedIcon.new("/usr/lib/unity-lens-web-history/globe.svg"),
                                       Unity.CategoryRenderer.HORIZONTAL_TILE))
        self.lens.props.categories = cats

        # Populate filters
        filters = []
        filter1 = Unity.RadioOptionFilter.new("lastvisited",
                                              _("Last Visited"),
                                              Gio.ThemedIcon.new("input-keyboard-symbolic"),
                                              False)
        filter1.add_option("lastday", _("Last Day"), None)
        filter1.add_option("lastweek", _("Last Week"), None)
        filter1.add_option("lastmonth", _("Last Month"), None)
        filter1.add_option("lastyear", _("Last Year"), None)
        filters.append(filter1)
        self.lens.props.filters = filters

        # Listen for changes and requests
        self.scope.connect("search-changed", self.on_search_changed)
        self.scope.connect("filters-changed", self.on_filters_changed)
        self.scope.connect("notify::active", self.on_lens_active)
        self.scope.connect("activate-uri", self.activate_uri)
        self.lens.add_local_scope(self.scope)
        self.lens.export()

    def on_filters_changed(self, *_):
        """ Called when a filter is clicked.  Queue's a new search
        """
        self.scope.queue_search_changed(Unity.SearchType.DEFAULT)

    def on_lens_active(self, *_):
        """ Called when the lens is activated.  Queue's a new search
        """
        if self.scope.props.active:
            self.scope.queue_search_changed(Unity.SearchType.DEFAULT)

    def on_search_changed(self, scope, search=None, search_type=0, cancellable=None):
        """ Called when the search is changed.  Gets the search string and search
        type and passes it to update_results_model()
        Args:
          scope: a scope object
          search: the search string
          search_type: the search type(global or local)
          cancellable: cancellable object
        """
        if hasattr(search, "props"):
            search_string = search.props.search_string
        else:
            search_string = ""

        if search_type == Unity.SearchType.DEFAULT:
            results = scope.props.results_model
        else:
            results = scope.props.global_results_model

        print "Search changed to: '%s'" % search_string
        self.update_results_model(search_string, results)
        if hasattr(search, "finished"):
            search.finished()

    def update_results_model(self, search, model):
        """ Clears the model of existing results, then performs a Zeitgeist search for the terms
        for the most recent websites visited and the most popular websites visited
        Args:
          search: the search terms
          model: the results model
        """
        model.clear()
        event_template = datamodel.Event()
        interpretation = getattr(datamodel.Interpretation, "WEBSITE")

        if search == '':
            count = 20
        else:
            count = 1000

        event_template.append_subject(datamodel.Subject.new_for_values(interpretation=interpretation))
        self.find_events(event_template, count, "mostrecent", GROUP_MOSTRECENT, search, model)
        self.find_events(event_template, count, "mostpopular", GROUP_MOSTPOPULAR, search, model)

    def find_events(self, event_template, total, sort, group, search, model):
        """ Searches Zeitgeist for events and adds them to the model
        Args:
          event_template: a zeitgeist template for the event
          total: number of events to return from Zeitgeist
          sort: the sort of the events(most recent or most popular)
          group: the category to add the results to
          search: the search terms
          model: the results model
        """
        try:
            iface = client.ZeitgeistDBusInterface()
        except RuntimeError:
            print "Could not connect to Zeitgeist."
            sys.exit(1)

        filters = self.scope.get_filter("lastvisited")
        if filters is not None:
            option = filters.get_active_option()
            if option is not None:
                timerange = option.props.id
            else:
                timerange = "all"

        if timerange == "lastday":
            time = datamodel.TimeRange.from_seconds_ago(24 * 60 * 60)
        elif timerange == "lastweek":
            time = datamodel.TimeRange.from_seconds_ago(7 * 24 * 60 * 60)
        elif timerange == "lastmonth":
            time = datamodel.TimeRange.from_seconds_ago(30 * 24 * 60 * 60)
        elif timerange == "lastyear":
            time = datamodel.TimeRange.from_seconds_ago(365 * 24 * 60 * 60)
        else:
            time = datamodel.TimeRange.until_now()

        if sort == "mostrecent":
            resulttype = datamodel.ResultType.MostRecentSubjects
        if sort == "mostpopular":
            resulttype = datamodel.ResultType.MostPopularSubjects

        #Search Zeitgeist for events
        events = iface.FindEvents(time,
                                  [event_template, ],
                                  datamodel.StorageState.Any,
                                  total,
                                  resulttype)
        events = (datamodel.Event(event) for event in events)
        self.i = 0

        if model == self.scope.props.global_results_model:
            group = GROUP_INTERNET

        for event in events:
            for subject in event.get_subjects():
                #Search the event's title for the search text
                if subject.uri.lower().find(search.lower()) == -1:
                    pass
                else:
                    if self.i < MAX:
                        uri = subject.uri
                        uri = urlparse.urlparse(uri)
                        icon_hint = uri.scheme + "://" + uri.netloc + "/favicon.ico"

                        model.append(subject.uri,       # uri
                                     icon_hint,         # string formatted GIcon
                                     group,             # numeric group id
                                     subject.mimetype,  # mimetype
                                     subject.text,      # display name
                                     subject.uri,       # comment
                                     subject.uri)
                        self.i = self.i + 1
                        continue

                if subject.text.lower().find(search.lower()) == -1:
                    pass
                else:
                    if self.i < MAX:
                        uri = subject.uri
                        uri = urlparse.urlparse(uri)
                        icon_hint = uri.scheme + "://" + uri.netloc + "/favicon.ico"

                        model.append(subject.uri,       # uri
                                     icon_hint,         # string formatted GIcon
                                     group,             # numeric group id
                                     subject.mimetype,  # mimetype
                                     subject.text,      # display name
                                     subject.uri,       # comment
                                     subject.uri)
                        self.i = self.i + 1

    def activate_uri(self, scope, uri):
        """ Open the url in the default webbrowser
        Args:
          uri: The url to be opened
        """
        webbrowser.open(uri)
        return Unity.ActivationResponse(handled=Unity.HandledType.HIDE_DASH, goto_uri='')

if __name__ == "__main__":
    SESSION_BUS_CONNECTION = Gio.bus_get_sync(Gio.BusType.SESSION, None)
    SESSION_BUS = Gio.DBusProxy.new_sync(SESSION_BUS_CONNECTION, 0, None,
                                         'org.freedesktop.DBus',
                                         '/org/freedesktop/DBus',
                                         'org.freedesktop.DBus', None)
    RESULT = SESSION_BUS.call_sync('RequestName',
                                   GLib.Variant("(su)", (BUS_NAME, 0x4)),
                                   0, -1, None)
    RESULT = RESULT.unpack()[0]

    if RESULT != 1:
        print >> sys.stderr, "Failed to own name %s. Bailing out." % BUS_NAME
        raise SystemExit(1)
    DAEMON = Daemon()
    GObject.MainLoop().run()
